#include <rtems.h>
#include <libcpu/io.h>
#include <libcpu/stackTrace.h>

#include <inttypes.h>
#include <stdlib.h>

#define CONFIGURE_INIT
#include "init.h"

#include "mvme3100_timer.h"


static const uint32_t max_wait_cycles = 4096; // Arbitrary


rtems_task Init(rtems_task_argument arg)
{
	struct mvme3100_timer timer;


	if (mvme3100_timer_init(&timer) != 0)
		rtems_fatal_error_occurred(0);


	srand(time(NULL));


	printk("Starting...\n");

	while (1)
	{
		if (mvme3100_timer_wait(&timer,
#ifdef _POSIX_MONOTONIC_CLOCK
				                CLOCK_MONOTONIC,
#else
				                CLOCK_REALTIME,
#endif
                                rand() % (max_wait_cycles + 1)) != 0)
		{
			rtems_fatal_error_occurred(-1);
		}
	}
}


void fatal_extension(Internal_errors_Source the_source,
                     bool                   is_internal,
                     Internal_errors_t      the_error)
{
	uint32_t isr_cookie;


//	mvme3100_led_fail_set(true);


	printk("Fatal %sinternal error: Source: %u, error: %u. Stack trace:\n", is_internal ? "" : "non-", the_source, the_error);
	CPU_print_stack(); //!< \todo Document in the Detailed Design that this is not only RTEMS-specific, but RTEMS-on-PowerPC-specific


	// We can't call _CPU_Fatal_halt() as it'll end up calling us again, so halt ourselves
	//! \bug Not SMP safe
	_CPU_ISR_Disable(isr_cookie);

	(void) isr_cookie; // Shut GCC up about isr_cookie being set but not used. We don't care


	while (1)
		;
}

