#ifndef __MVME3100_TIMER_H
#define __MVME3100_TIMER_H


#include <rtems.h>

#include <stddef.h>
#include <time.h>


//! \brief "Opaque" type representing an MVME3100 timer instance
struct mvme3100_timer
{
	size_t index; //!< When valid, uniquely identifies a timer

	rtems_id semaphore;
};


//! Behaviour is undefined if called with another thread in mvme3100_timer_wait()
int mvme3100_timer_destroy(struct mvme3100_timer *timer);

//! \brief Initialises an MVME3100 timer
//!
//!
//! \param timer Initialised to represent the allocated timer
//!
//! \return 0 on success, or non-zero with \p errno set to indicate the type of failure
int mvme3100_timer_init(struct mvme3100_timer *timer);

//! \brief Blocks on a timer \p timer for at least the period specified in nanoseconds in \p ns
//!
//!
//! \param timer Initialised to represent the allocated timer
//! \param clock The clock \p end relates to
//! \param end   The absolute time the delay should occur until
//!
//! \return 0 on success, or non-zero with \p errno set to indicate the type of failure
int mvme3100_timer_wait(struct mvme3100_timer *timer,
                        clockid_t             clock_id,
                        uint32_t              openpic_timer_cycles);


#endif // __MVME3100_TIMER_H
